"""
TBD
"""

__author__ = "Lukas Mahler"
__version__ = "0.0.1"
__date__ = "20.03.2022"
__email__ = "m@hler.eu"
__status__ = "Development"


# Default
import sys
import json
import time
import inspect

# Custom
import ts3
import requests

# Self
from src import util


def annoy(self, invkr_id, parameter):
    """

    """

    if not len(parameter) != 1:
        err = "Please use the command like this: .annoy TARGET MESSAGE"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return

    target = parameter[0]
    msg = parameter[1]
    self.poke(n=10, msg=msg, usr=target)


def follow(self, invkr_id, parameter):
    """
    TODO sticky folgen
    """

    if not parameter:
        err = "Please use the command like this: .follow TARGET"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return

    target = parameter[0]
    clid = self.identifytarget(target)
    cid = self.bot.clientinfo(clid=int(clid))[0]["cid"]
    self.bot.clientmove(clid=self.myid, cid=cid)


def help(self, invkr_id):
    """

    """

    # Find all functions in this submodule
    cmds = [f[0] for f in inspect.getmembers(sys.modules[__name__], inspect.isfunction)]
    msg = f"List of commands:\n---------------------\n.{f'{chr(10)}.'.join(cmds)}"
    self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=msg)


def info(self, invkr_id):
    """

    """

    msg = f"Runtime: {util.getRuntime(self.started)}\n" \
          f"Version: {self.version}"
    self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=msg)


def list(self, invkr_id, parameter):
    """
    Message the invoker of the function either a list of channels, clients, commands or server groups.
    """

    if not parameter:
        err = "Please use the command like this: .list channel/clients/commands/groups/"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return

    what = parameter[0]
    mydict = {}

    if what == "channel":
        channels = self.bot.channellist()
        for channel in channels:
            order = channel["channel_order"]
            mydict[order] = [channel["channel_name"], channel["cid"]]

        mydict = dict(sorted(mydict.items()))
        msg = json.dumps(mydict)

    elif what == "clients":
        clients = self.bot.clientlist()
        for client in clients:
            mydict[client["client_nickname"]] = {"clid": client["clid"], "cldbid": client["client_database_id"]}
        mydict = dict(sorted(mydict.items()))
        msg = json.dumps(mydict)

    elif what == "commands":
        help(self, invkr_id)
        return

    elif what == "groups":
        groups = self.bot.servergrouplist()
        for group in groups:
            mydict[group["name"]] = {"sgid": group["sgid"], "sortid": group["sortid"]}
        msg = json.dumps(mydict)

    else:
        msg = f"The parameter [{what}] is not supported."

    self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=msg)


def mute(self, invkr_id, parameter):
    """
    Assign the mute group to a user.
    """

    if not parameter:
        err = "Please use the command like this: .mute TARGET"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return

    target = parameter[0]
    clid = self.identifytarget(target)
    cldbid = self.clid2cldbid(clid)

    try:
        self.bot.servergroupaddclient(sgid=self.sgid_mute, cldbid=cldbid)
    except ts3.query.TS3QueryError as e:
        err = f"Failed to add to group: [{e}]"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return


def pingall(self, invkr_id, parameter):
    """

    """

    if not parameter:
        err = "Please use the command like this: .pingall MESSAGE"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return

    msg = parameter[0]
    self.poke(msg=msg)


def notifyAdmin(self):
    """
    Message every available admin.
    """

    clients = self.bot.clientlist()
    clients = [client for client in clients if client["client_type"] != "1"]
    for client in clients:
        cldbid = client["client_database_id"]
        clid = client["clid"]
        if self.isadmin(cldbid):
            msg = "<Keep this chat open to use commands>"
            self.bot.sendtextmessage(targetmode=1, target=clid, msg=msg)

        if not self.whitelisted:
            time.sleep(1)


def quit(self, invkr_id):
    """

    """
    self.stop(invkr_id)


def rename(self, invkr_id, parameter):
    """
    Rename the Bot to the given Nickname
    """

    if not parameter:
        err = "Please use the command like this: .rename NICKNAME"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return

    self.nickname = parameter[0]
    self.bot.clientupdate(client_nickname=self.nickname)


def rwf(self):
    """
    Refresh the WoW progress channel,
    this is very specific to my use, you can disable this in the config.
    """
    url = "https://raider.io/api/v1/raiding/progression?raid=sepulcher-of-the-first-ones&difficulty=mythic&region=world"
    data = json.loads(requests.get(url).text)

    prog_done = 0
    for entry in data['progression']:
        if entry['totalGuilds'] > 0:
            prog_done += 1

    prog_min = prog_done
    prog_max = len(data['progression'])
    channelname = f"[cspacerC3]Sepulcher {prog_min}/{prog_max}"

    try:
        self.editChannelname(205, channelname)
    except ts3.query.TS3QueryError:
        pass


def ticker(self, parameter):
    """
    Refresh the Crypto Ticker channels,
    this is very specific to my use, you can disable this in the config.
    """

    if parameter:
        symbol = parameter[0].lower()
    else:
        symbol = "all"

    if symbol in ["btc", "bitcoin", "eth", "ethereum", "all"]:
        channelname = f"[cspacerC1]BTC: {self.gecko.getSymbol('bitcoin', decimal=0)} | " \
                      f"ETH: {self.gecko.getSymbol('ethereum', decimal=0)}"
        try:
            self.editChannelname(200, channelname)
        except ts3.query.TS3QueryError:
            pass

    if symbol in ["dot", "polkadot", "ada", "cardano", "all"]:
        channelname = f"[cspacerC2]DOT: {self.gecko.getSymbol('polkadot', decimal=2)} | " \
                      f"ADA: {self.gecko.getSymbol('cardano', decimal=2)}"
        try:
            self.editChannelname(201, channelname)
        except ts3.query.TS3QueryError:
            pass


def unmute(self, invkr_id, parameter):
    """
    Remove the mute group to a user.
    """

    if not parameter:
        err = "Please use the command like this: .unmute TARGET"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return

    target = parameter[0]
    clid = self.identifytarget(target)
    cldbid = self.clid2cldbid(clid)

    try:
        self.bot.servergroupdelclient(sgid=self.sgid_mute, cldbid=cldbid)
    except ts3.query.TS3QueryError as e:
        err = f"Failed to remove from group: [{e}]"
        self.bot.sendtextmessage(targetmode=1, target=invkr_id, msg=err)
        return


if __name__ == "__main__":
    exit()
